'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
	value: true
});

var _path2 = require('path');

var _path3 = _interopRequireDefault(_path2);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _toolsNodeHook = require('./tools/node-hook');

var _toolsNodeHook2 = _interopRequireDefault(_toolsNodeHook);

var _toolsSerializeJavascript = require('./tools/serialize-javascript');

var _toolsSerializeJavascript2 = _interopRequireDefault(_toolsSerializeJavascript);

var _toolsLog = require('./tools/log');

var _toolsLog2 = _interopRequireDefault(_toolsLog);

var _helpers = require('./helpers');

var _common = require('./common');

// using ES6 template strings
// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/template_strings

var webpack_isomorphic_tools = (function () {
	function webpack_isomorphic_tools(options) {
		_classCallCheck(this, webpack_isomorphic_tools);

		// take the passed in options
		this.options = (0, _helpers.alias_camel_case)((0, _helpers.clone)(options));

		// used to keep track of cached assets and flush their caches on .refresh() call
		this.cached_assets = [];

		// add missing fields, etc
		(0, _common.normalize_options)(this.options);

		// logging
		this.log = new _toolsLog2['default']('webpack-isomorphic-tools', { debug: this.options.debug });

		this.log.debug('instantiated webpack-isomorphic-tools with options', this.options);
	}

	_createClass(webpack_isomorphic_tools, [{
		key: 'development',

		// sets development mode flag to whatever was passed (or true if nothing was passed)
		// (development mode allows asset hot reloading when used with webpack-dev-server)
		value: function development(flag) {
			// set development mode flag
			this.options.development = (0, _helpers.exists)(flag) ? flag : true;

			if (this.options.development) {
				this.log.debug('entering development mode');
			} else {
				this.log.debug('entering production mode');
			}

			// allows method chaining
			return this;
		}
	}, {
		key: 'assets',

		// returns a mapping to read file paths for all the user specified asset types
		// along with a couple of predefined ones: javascripts and styles
		value: function assets() {
			// when in development mode
			if (this.options.development) {
				// webpack and node.js start in parallel
				// so webpack-assets.json might not exist on the very first run
				// if a developer chose not to use the .server() method with a callback
				// (or if a developer chose not to wait for a Promise returned by the .server() method)
				if (!_fs2['default'].existsSync(this.webpack_assets_path)) {
					this.log.error('"' + this.webpack_assets_path + '" not found. Most likely it hasn\'t yet been generated by Webpack. Using an empty stub instead.');
					return (0, _common.default_webpack_assets)();
				}
			}

			return require(this.webpack_assets_path);
		}
	}, {
		key: 'refresh',

		// clear the require.cache (only used in developer mode with webpack-dev-server)
		value: function refresh() {
			// ensure this is development mode
			if (!this.options.development) {
				throw new Error('.refresh() called in production mode. Did you forget to call .development() method on your webpack-isomorphic-tools server instance?');
			}

			this.log.debug('flushing require() caches');

			// uncache webpack-assets.json file
			// this.log.debug(' flushing require() cache for webpack assets json file')
			// this.log.debug(` (was cached: ${typeof(require.cache[this.webpack_assets_path]) !== 'undefined'})`)
			delete require.cache[this.webpack_assets_path];

			// uncache cached assets
			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = _getIterator(this.cached_assets), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var _path = _step.value;

					this.log.debug(' flushing require() cache for ' + _path);
					delete require.cache[_path];
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator['return']) {
						_iterator['return']();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}

			// no assets are cached now
			this.cached_assets = [];
		}
	}, {
		key: 'server',

		// Initializes server-side instance of `webpack-isomorphic-tools`
		// with the base path for your project, then calls `.register()`,
		// and after that calls .wait_for_assets(callback).
		//
		// The `project_path` parameter must be identical
		// to the `context` parameter of your Webpack configuration
		// and is needed to locate `webpack-assets.json`
		//  which is output by Webpack process.
		//
		// sets up "project_path" option
		// (this option is required on the server to locate webpack-assets.json)
		value: function server(project_path, callback) {
			var _this = this;

			// project base path, required to locate webpack-assets.json
			this.options.project_path = project_path;

			// resolve webpack-assets.json file path
			this.webpack_assets_path = _path3['default'].resolve(this.options.project_path, this.options.webpack_assets_file_path);

			// register require() hooks
			this.register();

			// when ready:

			// if callback is given, call it back
			if (callback) {
				// call back when ready
				return this.wait_for_assets(callback);
			}
			// otherwise resolve a Promise
			else {
				// no callback given, return a Promise
				return new _Promise(function (resolve, reject) {
					return _this.wait_for_assets(resolve);
				});
			}
		}
	}, {
		key: 'register',

		// Registers Node.js require() hooks for the assets
		//
		// This is what makes the `requre()` magic work on server.
		// These `require()` hooks must be set before you `require()`
		// any of your assets
		// (e.g. before you `require()` any React components
		// `require()`ing your assets).
		//
		// read this article if you don't know what a "require hook" is
		// http://bahmutov.calepin.co/hooking-into-node-loader-for-fun-and-profit.html
		value: function register() {
			var _this2 = this;

			this.log.debug('registering require() hooks for assets');

			// for each user specified asset type
			_Object$keys(this.options.assets)

			// // which isn't for .js files
			// // (because '.js' files requiring already works natively)
			// .filter(asset_type =>
			// {
			// 	const description = this.options.assets[asset_type]
			//
			// 	if (description.extension)
			// 	{
			// 		return description.extension !== 'js'
			// 	}
			// 	else
			// 	{
			// 		return description.extensions.indexOf('js') < 0
			// 	}
			// })

			// register a require hook for each file extension of this asset type
			.forEach(function (asset_type) {
				var description = _this2.options.assets[asset_type];

				var _iteratorNormalCompletion2 = true;
				var _didIteratorError2 = false;
				var _iteratorError2 = undefined;

				try {
					for (var _iterator2 = _getIterator(description.extensions), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
						var extension = _step2.value;

						_this2.register_extension(extension, description);
					}
				} catch (err) {
					_didIteratorError2 = true;
					_iteratorError2 = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion2 && _iterator2['return']) {
							_iterator2['return']();
						}
					} finally {
						if (_didIteratorError2) {
							throw _iteratorError2;
						}
					}
				}
			});

			// allows method chaining
			return this;
		}
	}, {
		key: 'register_extension',

		// registers a require hook for a particular file extension
		value: function register_extension(extension, description) {
			var _this3 = this;

			this.log.debug(' registering a require() hook for *.' + extension);

			// place the require() hook for this extension
			_toolsNodeHook2['default'].hook('.' + extension, function (path, fallback) {
				return _this3.require(path, description, fallback);
			});
		}
	}, {
		key: 'require',

		// require()s an asset by a path
		value: function require(global_asset_path, description, fallback) {
			this.log.debug('require() called for ' + global_asset_path);

			// sanity check
			if (!this.options.project_path) {
				throw new Error('You forgot to call the .server() method passing it your project\'s base path');
			}

			// // if the require()d file is not part of the project - skip it
			// // (should be faster, but who needs faster if the modules are cached by Node.js)
			// if (!global_asset_path.indexOf(this.options.project_path) === 0)
			// {
			// 	this.log.debug(` skipping require call for ${asset_path} (not in project folder)`)
			// 	return fallback()
			// }

			// // asset path relative to the project folder
			// // (should be faster, but who needs faster if the modules are cached by Node.js)
			// let asset_path = global_asset_path.slice(this.options.project_path.length + fs.sep.length)

			// asset path relative to the project folder
			var asset_path = _path3['default'].relative(this.options.project_path, global_asset_path);

			// for Windows:
			//
			// convert Node.js path to a correct Webpack path
			asset_path = asset_path.replace(/\\/g, '/');
			// add './' in the beginning if it's missing (for example, in case of Windows)
			if (asset_path.indexOf('.') !== 0) {
				asset_path = './' + asset_path;
			}

			// if this filename is in the user specified exceptions list
			// (or is not in the user explicitly specified inclusion list)
			// then fallback to the normal require() behaviour
			if (!this.includes(asset_path, description) || this.excludes(asset_path, description)) {
				this.log.debug(' skipping require call for ' + asset_path);
				return fallback();
			}

			// track cached assets (only in development mode)
			if (this.options.development) {
				// mark this asset as cached
				this.cached_assets.push(global_asset_path);
			}

			// webpack has a shortcut from "node_modules"
			if (asset_path.indexOf('./node_modules/') === 0) {
				asset_path = asset_path.replace('./node_modules/', './~/');
			}

			// perform the actual require() of this asset
			return this._require(asset_path);
		}
	}, {
		key: '_require',

		// is called when you require() your assets
		// (or can be used manually without require hooks)
		value: function _require(asset_path) {
			this.log.debug(' requiring ' + asset_path);

			// sanity check
			if (!asset_path) {
				return undefined;
			}

			// get real file path list
			var assets = this.assets().assets;

			// find this asset in the real file path list
			var asset = assets[asset_path];

			// if the real path was found in the list - return it
			if ((0, _helpers.exists)(asset)) {
				return asset;
			}

			// serve a not-found asset maybe
			this.log.error('asset not found: ' + asset_path);
			return undefined;
		}
	}, {
		key: 'excludes',

		// Checks if the required path should be excluded from the custom require() hook
		value: function excludes(path, options) {
			// if "exclude" parameter isn't specified, then exclude nothing
			if (!(0, _helpers.exists)(options.exclude)) {
				return false;
			}

			// for each exclusion case
			var _iteratorNormalCompletion3 = true;
			var _didIteratorError3 = false;
			var _iteratorError3 = undefined;

			try {
				for (var _iterator3 = _getIterator(options.exclude), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
					var exclude = _step3.value;

					// supports regular expressions
					if (exclude instanceof RegExp) {
						if (exclude.test(path)) {
							return true;
						}
					}
					// check for a compex logic match
					else if (typeof exclude === 'function') {
						if (exclude(path)) {
							return true;
						}
					}
					// otherwise check for a simple textual match
					else {
						if (exclude === path) {
							return true;
						}
					}
				}
			} catch (err) {
				_didIteratorError3 = true;
				_iteratorError3 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion3 && _iterator3['return']) {
						_iterator3['return']();
					}
				} finally {
					if (_didIteratorError3) {
						throw _iteratorError3;
					}
				}
			}

			// no matches found.
			// returns false so that it isn't undefined (for testing purpose)
			return false;
		}
	}, {
		key: 'includes',

		// Checks if the required path should be included in the custom require() hook
		value: function includes(path, options) {
			// if "include" parameter isn't specified, then include everything
			if (!(0, _helpers.exists)(options.include)) {
				return true;
			}

			// for each inclusion case
			var _iteratorNormalCompletion4 = true;
			var _didIteratorError4 = false;
			var _iteratorError4 = undefined;

			try {
				for (var _iterator4 = _getIterator(options.include), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
					var include = _step4.value;

					// supports regular expressions
					if (include instanceof RegExp) {
						if (include.test(path)) {
							return true;
						}
					}
					// check for a compex logic match
					else if (typeof include === 'function') {
						if (include(path)) {
							return true;
						}
					}
					// otherwise check for a simple textual match
					else {
						if (include === path) {
							return true;
						}
					}
				}
			} catch (err) {
				_didIteratorError4 = true;
				_iteratorError4 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion4 && _iterator4['return']) {
						_iterator4['return']();
					}
				} finally {
					if (_didIteratorError4) {
						throw _iteratorError4;
					}
				}
			}

			// no matches found.
			// returns false so that it isn't undefined (for testing purpose)
			return false;
		}
	}, {
		key: 'wait_for_assets',

		// Waits for webpack-assets.json to be created after Webpack build process finishes
		//
		// The callback is called when `webpack-assets.json` has been found
		// (it's needed for development because `webpack-dev-server`
		//  and your application server are usually run in parallel).
		//
		value: function wait_for_assets(done) {
			var _this4 = this;

			// condition check interval
			var interval = 300; // in milliseconds

			// selfie
			var tools = this;

			// waits for condition to be met, then proceeds
			function wait_for(condition, proceed) {
				function check() {
					// if the condition is met, then proceed
					if (condition()) {
						return proceed();
					}

					tools.log.debug('(' + tools.webpack_assets_path + ' not found)');
					tools.log.info('(waiting for the first Webpack build to finish)');

					setTimeout(check, interval);
				}

				check();
			}

			// wait for webpack-assets.json to be written to disk by Webpack
			wait_for(function () {
				return _fs2['default'].existsSync(_this4.webpack_assets_path);
			}, done);

			// allows method chaining
			return this;
		}
	}]);

	return webpack_isomorphic_tools;
})();

exports['default'] = webpack_isomorphic_tools;
module.exports = exports['default'];
//# sourceMappingURL=index.js.map